<?php

namespace JimsCustomerBooking\Generators;

if (!class_exists('JimsCustomerBooking\Generators\CBURLGenerator')) {
    /**
     * its construct should get an array from JimsOptions getProperties method output.
     */
    class CBURLGenerator extends Generator
    {
        protected $values = [];

        /**
         * @param mixed $is_iframe
         *
         * @return string
         */
        public function generate($is_iframe = true)
        {
            if ($is_iframe) {
                $this->setParent();
            }

            return sprintf('%s?%s', $this->getBaseUrl(), $this->getPropertiesString());
        }

        /**
         * @return string
         */
        public function getBaseUrl()
        {
            return "{$this->http_schema}://{$this->domain}/";
        }

        /**
         * @param mixed $excluded
         *
         * @return string
         */
        public function getPropertiesString($excluded = ['http_schema', 'domain', 'cssSelector', 'redirectOnly'])
        {
            $values = $this->shortProperties($this->filterConflictingProperties($this->values));

            $properties = [];
            foreach ($values as $field => $value) {
                if (!in_array($field, $excluded) && !empty($value)) {
                    $properties[$field] = str_replace('#', '', $value);
                }
            }

            return http_build_query($properties);
        }

        public function setParent()
        {
            $this->values['parent'] = get_bloginfo('url');
        }

        /**
         * @param mixed $properties
         *
         * @return mixed
         */
        public function filterConflictingProperties($properties)
        {
            if (!empty($properties['gatId']) && !empty($properties['gtmId'])) {
                unset($properties['gatId']);
            }

            return $properties;
        }

        /**
         * Short Properties in order to have DivisionId and gtmId or gatId first
         * after these the rest.
         *
         * @param $properties array
         *
         * @return array
         */
        public function shortProperties($properties)
        {
            $shorted_property_names = ['DivisionId', 'gtmId', 'gatId'];
            $shorted = [];

            foreach ($shorted_property_names as $name) {
                if (isset($properties[$name]) || isset($properties[strtolower($name)])) {
                    $shorted[$name] = $properties[$name];
                    unset($properties[$name]);
                }
            }

            return array_merge($shorted, $properties);
        }
    }
}
