<?php

namespace JimsCustomerBooking\Helpers;

class ColorHelper extends Helper
{
    /**
     * @param string $hex
     */
    public static function hexToRgb($hex)
    {
        $hex = str_replace('#', '', $hex);

        if (strlen($hex) == 3) {
            $r = hexdec(substr($hex, 0, 1) . substr($hex, 0, 1));
            $g = hexdec(substr($hex, 1, 1) . substr($hex, 1, 1));
            $b = hexdec(substr($hex, 2, 1) . substr($hex, 2, 1));
        } else {
            $r = hexdec(substr($hex, 0, 2));
            $g = hexdec(substr($hex, 2, 2));
            $b = hexdec(substr($hex, 4, 2));
        }

        return [$r, $g, $b];
    }

    public static function rgbToHex(int $red, int $green, int $blue)
    {
        $red = dechex($red);
        $green = dechex($green);
        $blue = dechex($blue);

        return '#' . $red . $green . $blue;
    }

    /**
     * @param int   $percent
     * @param mixed $returnAsString
     */
    public static function darken(string $hex, $percent = 10, $returnAsString = true)
    {
        $rgb = self::hexToRgb($hex);
        $r = $rgb[0];
        $g = $rgb[1];
        $b = $rgb[2];

        $r = (int) round($r * (100 - $percent) / 100);
        $g = (int) round($g * (100 - $percent) / 100);
        $b = (int) round($b * (100 - $percent) / 100);

        return $returnAsString ? self::rgbToHex($r, $g, $b) : [$r, $g, $b];
    }

    /**
     * @param int $percent
     */
    public static function lighten(string $hex, $percent = 10)
    {
        $rgb = self::hexToRgb($hex);
        $r = $rgb[0];
        $g = $rgb[1];
        $b = $rgb[2];

        $r = (int) round($r * (100 + $percent) / 100);
        $g = (int) round($g * (100 + $percent) / 100);
        $b = (int) round($b * (100 + $percent) / 100);

        return self::rgbToHex($r, $g, $b);
    }
}
