<?php

namespace JimsCustomerBooking\Helpers;

use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\JimsOptions;
use JimsCustomerBooking\JimsPlugin;
use JimsCustomerBooking\Services\TokenManager;

if (!class_exists('JimsCustomerBooking\Helpers\OptionHelper')) {
    class OptionHelper extends Helper
    {
        /**
         * @param mixed      $option_name
         * @param null|mixed $default
         *
         * @return false|mixed|void
         */
        public static function get($option_name, $default = null)
        {
            return get_option(StringHelper::getPrefixed($option_name, 'machine'), $default);
        }

        public static function update($option_name, $value): bool
        {
            return update_option(StringHelper::getPrefixed($option_name, 'machine'), $value);
        }

        public static function getValue($option_name, $value_name, $default = null)
        {
            $option = self::get($option_name, []);

            return $option[$value_name] ?? $default;
        }

        /**
         * @return JimsOptions
         */
        public static function getEmbeddedFormOptions()
        {
            $default = new JimsOptions();
            $options = self::get('form', $default);

            if ($options instanceof JimsOptions) {
                return $options;
            }

            if (is_string($options)) {
                $options = \unserialize($options);
            }

            return $options instanceof JimsOptions ? $options : $default;
        }

        public static function localizeFormVariables($script_name)
        {
            $settings = static::get(NativeFormSettingsPage::$option_name, []);

            // remove options with key starting with 'required_fields'
            $settings = array_filter($settings, function ($key) {
                return !str_contains($key, 'required_fields');
            }, ARRAY_FILTER_USE_KEY);

            $settings['requiredFields'] = self::getRequiredInfoFields();

            wp_localize_script(
                StringHelper::getPrefixed($script_name),
                StringHelper::getPrefixed('variables', 'machine'),
                array_merge(
                    [
                        'home_url' => get_bloginfo('wpurl'),
                        'version' => JimsPlugin::$version,
                        'wpcb_url' => JIMS_CBP_URL,
                        'rest_url' => get_rest_url(),
                        'secret' => wp_create_nonce('wp_rest'),
                        'session_token' => TokenManager::encode(
                            JimsPlugin::SECKEY,
                            JimsPlugin::PREFIX . TokenManager::generateRandomString(10),
                            '10 minutes'
                        ),
                    ],
                    $settings,
                    ['embeddedFormOptions' => JimsPlugin::getOptions()->getProperties()],
                )
            );
        }

        public static function getArrayFirstValue(array $arr)
        {
            return array_values($arr)[0] ?? null;
        }

        public static function getFormOption($name, $default = '')
        {
            $options = self::get(NativeFormSettingsPage::$option_name, []);

            return $options[$name] ?? $default;
        }

        /**
         * @param string $target, available options are 'Internal', 'FMS'
         */
        public static function isHTTPCompatibilityEnabled(string $target = 'FMS'): bool
        {
            if (in_array($target, ['Internal', 'FMS'])) {
                return (int) self::getFormOption('httpVersionFor' . $target, 0) === 1;
            }

            return false;
        }

        public static function getRequiredInfoFields()
        {
            $options = OptionHelper::get(NativeFormSettingsPage::$option_name, []);

            $fields = array_filter(array_keys($options), function ($field) {
                return str_contains($field, 'required_fields');
            });

            $fields[] = 'address';

            if (count($fields) <= 1) {
                return [
                    'address',
                    'firstname',
                    'lastname',
                    'email',
                    'phone',
                ];
            }

            return array_map(function ($field) {
                return str_replace('required_fields-', '', $field);
            }, $fields);
        }
    }
}
