<?php

namespace JimsCustomerBooking;

use JimsCustomerBooking\Admin\BackendPages\CustomerBookingPage;
use JimsCustomerBooking\Admin\BackendPages\EmailSubscriptionSettingsPage;
use JimsCustomerBooking\Admin\BackendPages\FormCustomizeSettingsPage;
use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Admin\BackendPages\SettingsPage;
use JimsCustomerBooking\Admin\BackendPages\ShortCodeGeneratorPage;
use JimsCustomerBooking\Database\Database;
use JimsCustomerBooking\Helpers\ColorHelper;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Integrations\JimsLeadsIntegration;
use JimsCustomerBooking\Middlewares\Http;
use JimsCustomerBooking\RestfulAPI\AuthenticationController;
use JimsCustomerBooking\RestfulAPI\BookingsStatisticsController;
use JimsCustomerBooking\RestfulAPI\CustomerBookingController;
use JimsCustomerBooking\RestfulAPI\CustomerBookingCustomizeController;
use JimsCustomerBooking\RestfulAPI\EmailSubscriptionController;
use JimsCustomerBooking\RestfulAPI\InputValidationController;
use JimsCustomerBooking\RestfulAPI\OptionController;
use JimsCustomerBooking\RestfulAPI\ScheduleController;
use JimsCustomerBooking\RestfulAPI\ServiceController;
use JimsCustomerBooking\Shortcodes\BookingFormShortCode;
use JimsGroup\Utilities\Logger;

if (!class_exists('JimsCustomerBooking\JimsPlugin')) {
    class JimsPlugin
    {
        /**
         * --- PLEASE DO NOT CHANGE THIS ---
         * encryption key for the plugin
         * it should be a hexadecimal key, 136 characters long.
         */
        // phpcs:ignore
        public const ENCRYPTION_KEY = 'def000008eb640d6634a6d422b06ecbbd8ed8bd0fc417c4831b7aa7bfb7c89a4caa3973a13288795670bbc0793fffa622cb5c99fa407069efb64a11a1b1d40716f3cd6c0';

        /**
         * key for the option in wordpress options table.
         */
        public const OPTION_NAME = 'wpcb';
        public const SECKEY = 'jims-wpcb-communication-key-v1';

        /**
         * --- PLEASE DO NOT CHANGE THIS ---
         * menu slug for admin.
         */
        public const PREFIX = 'jims_wpcb';

        public const FMS_API_KEY = '3e8bce9e-be45-4499-b31a-23b6fb067aa7';

        public const FMS_TOKENS = [
            'secret' => 'yw86Dvcwwsm0vEpZ3oNRFrXkCNovNLKh',
            'fso_token' => 'Zm1zX3VzZXI6Q2I2MkFZaFRwOFdtYUJSMnBzUmpSNUxTTnNhRWY1cEw=',
            'fse_token' => 'amltc19qb2JzX21vYmlsZTpDNHFkOFl0OGVnWVVLMnRaZnlhNWtaUjVZR3JmNng0eQ==',
        ];

        public const FINDAJIM_TOKEN = '84bcb06b2d534989cf811f250b26dc10';

        // local is for testing and available only for local environments
        public const FMS_API_MODES = [
            'production', 'uat', 'hf', 'staging', 'int', 'development', 'local',
        ];
        public static $version = '3.3.06';

        private array $rest_classes = [];

        /**
         * @ var array[]
         */
        private $registered = [];

        private ?JimsLeadsIntegration $leads = null;

        public function run(): void
        {
            add_action('plugins_loaded', [$this, 'activate']);
            add_action('init', [$this, 'setupShortcodes']);
            add_action('wp_enqueue_scripts', [$this, 'frontScripts']);
            add_action('admin_enqueue_scripts', [$this, 'adminScripts']);
            add_action('rest_api_init', [$this, 'registerRestAPI']);
            add_action( StringHelper::getPrefixed('jims_logo'), [$this, 'setupJimsLogo'] );
            add_action('admin_notices', [$this, 'showDashboardNotifications']);

            //modify shortcode block's callback
            add_filter('register_block_type_args', [$this, 'modifyShortcodeBlock'], 1, 2);


            // Checks the theme and theme version and modifies settings to match
            $this->setupCompatibility();
            $this->registerBackendPages();
            $this->leads = JimsLeadsIntegration::getInstance();
            $this->setupMiddleWares();
        }

        public function frontScripts(): void
        {
            $frontAsset = require JIMS_CBP_DIR . '/build/front.asset.php';

            wp_register_script(
                StringHelper::getPrefixed('js'),
                plugins_url('build/front.js', JIMS_CBP_FILE),
                $frontAsset['dependencies'],
                $frontAsset['version'],
                true
            );

            wp_register_style(StringHelper::getPrefixed('final'), false);

            list($main_color, $main_color_dim) = $this->getFormColors();

            wp_add_inline_style(
                StringHelper::getPrefixed('final'),
                \sprintf(':root { --nwpcb-main-color: %s; --nwpcb-main-color-dim: %s }', $main_color, $main_color_dim)
            );

            wp_enqueue_style(
                StringHelper::getPrefixed('style'),
                plugins_url('build/front.css', JIMS_CBP_FILE),
                [
                    StringHelper::getPrefixed('final'),
                ],
                $frontAsset['version'],
            );

            OptionHelper::localizeFormVariables('js');
            wp_enqueue_script(StringHelper::getPrefixed('js'));
        }

        public function adminScripts(): void
        {
            $htmlAsset = require JIMS_CBP_DIR . '/build/html-extension.asset.php';

            wp_enqueue_script(
                StringHelper::getPrefixed('html-extension'),
                plugins_url('build/html-extension.js', JIMS_CBP_FILE),
                $htmlAsset['dependencies'],
                $htmlAsset['version'],
            );

            $adminAsset = require JIMS_CBP_DIR . '/build/html-extension.asset.php';

            wp_register_script(
                StringHelper::getPrefixed('admin-js'),
                plugins_url('build/admin.js', JIMS_CBP_FILE),
                $adminAsset['dependencies'],
                $adminAsset['version'],
                true
            );

            OptionHelper::localizeFormVariables('admin-js');

            wp_enqueue_script(StringHelper::getPrefixed('admin-js'));

            wp_enqueue_style(
                StringHelper::getPrefixed('admin-style'),
                plugins_url('build/admin.css', JIMS_CBP_FILE),
                [],
                $adminAsset['version'],
            );

            // enqueue fontawesome if not theme is not the new jims theme
            $themeName = strtolower(wp_get_theme()->get('Name'));

            if (!str_contains($themeName, 'jims') || str_contains($themeName, 'enfold')) {
                wp_register_style(
                    StringHelper::getPrefixed('fontawesome-solid'),
                    plugins_url('build/fonts/css/solid.css', JIMS_CBP_FILE),
                    [],
                    $adminAsset['version'],
                );

                wp_enqueue_style(
                    StringHelper::getPrefixed('fontawesome-style'),
                    plugins_url('build/fonts/css/fontawesome.css', JIMS_CBP_FILE),
                    [
                        StringHelper::getPrefixed('fontawesome-solid'),
                    ],
                    $adminAsset['version'],
                );
            }
        }

        public function registerBackendPages(): array
        {
            return $this->registered['backend_pages'] = [
                SettingsPage::class => new SettingsPage(),
                NativeFormSettingsPage::class => new NativeFormSettingsPage(),
                FormCustomizeSettingsPage::class => new FormCustomizeSettingsPage(),
                EmailSubscriptionSettingsPage::class => new EmailSubscriptionSettingsPage(),
                CustomerBookingPage::class => new CustomerBookingPage(),
                ShortCodeGeneratorPage::class => new ShortCodeGeneratorPage(),
            ];
        }

        public function showDashboardNotifications(): void
        {
            if (!$this->isUserSubscribed()) {
                printf(
                    '<div class="notice notice-error is-dismissible"><p>%s <a href="%s">here</a></p></div>',
                    'WP CB form is <strong>LOCKED</strong>! To unlock please subscribe to our email service ',
                    admin_url('admin.php?page=wpcb_subscription_settings.php')
                );
            }

            if ($this->leads->getSiteStatus() === false) {
                printf(
                    '<div class="notice notice-error is-dismissible"><p>%s <a href="%s">%s</a> %s</p></div>',
                    // phpcs:ignore
                    'Your website appears to be actively blocking Jim\'s Group monitoring servers. This could potentially affect the health and error tracking of your CB form. Please ',
                    'mailto:infotech@jims.net',
                    'contact support',
                    'for more information.'
                );
            }
        }

        public function isUserSubscribed(): bool
        {
            $options = OptionHelper::get(EmailSubscriptionSettingsPage::$option_name, []);

            return isset($options['email'])
                && strlen($options['email']) > 3
                && str_contains($options['email'], '@');
        }

        public function setupShortcodes(): void
        {
            if ($this->isUserSubscribed()) {
                $this->registered['shortcodes'] = [
                    BookingFormShortCode::class => new BookingFormShortCode(),
                ];
            }
        }

        public function registerRestAPI(): void
        {
            $this->rest_classes = [
                InputValidationController::class => new InputValidationController(),
                OptionController::class => new OptionController(),
                ServiceController::class => new ServiceController(),
                AuthenticationController::class => new AuthenticationController(),
                CustomerBookingController::class => new CustomerBookingController(),
                BookingsStatisticsController::class => new BookingsStatisticsController(),
                CustomerBookingCustomizeController::class => new CustomerBookingCustomizeController(),
                EmailSubscriptionController::class => new EmailSubscriptionController(),
                ScheduleController::class => new ScheduleController(),
            ];

            // register rest api controllers hook
            $this->rest_classes = apply_filters(self::PREFIX . '_rest_controller_register', $this->rest_classes);
        }

        public function modifyShortcodeBlock($settings, $name): array
        {
            if ($name === 'core/shortcode') {
              $settings['render_callback'] = function ($attributes, $content) {
                  // check if the content has the prefix
                  if(str_contains($content, self::PREFIX)) {
                      return $content;
                  }

                  //if not, run the wpautop function
                  return function_exists('render_block_core_shortcode')
                      ? render_block_core_shortcode($attributes, $content)
                      : wpautop($content);
              };
            }

            return $settings;
        }

        public static function getOptions(): JimsOptions
        {
            return OptionHelper::getEmbeddedFormOptions();
        }

        public function activate(): void
        {
            $option_name = StringHelper::getPrefixed('version', 'machine');
            $current_version = get_option($option_name, '0.0.0');

            if ($current_version != JIMS_CBP_VERSION) {
                update_option($option_name, JIMS_CBP_VERSION, true);
                Database::setupOrUpdateDatabase($current_version);

                // update website on leads.jims.net
                $res = $this->updateWebsiteToLeads();
                Logger::info(is_wp_error($res) ? $res->get_error_message() : $res);
            }
        }

        public function setupCompatibility(): void
        {
            // check if enfold theme is active and get logo from theme options
            add_filter(StringHelper::getPrefixed('booking_form_logo_url'), function ($logo) {
                $theme = wp_get_theme();
                $theme_name = $theme->get('Name');

                if (str_contains(strtolower($theme_name), 'enfold')) {
                    $options = get_option('avia_options_jims_enfold');
                    $logo = isset($options['avia'], $options['avia']['logo']) ? $options['avia']['logo'] : '';
                }

                return $logo;
            }, 99);
        }

        public function getRegistered(string $name): array
        {
            return isset($this->registered[$name])
                ? $this->registered[$name]
                : $this->registered;
        }

        public function setupJimsLogo(): void
        {
            $plugin_url = plugins_url('build/img/jims.png', JIMS_CBP_FILE);
            printf('<img src="%s" alt="%s">', $plugin_url, 'Jims Logo');
        }

        public function getFormColors(): array
        {
            $main_color = OptionHelper::getValue(NativeFormSettingsPage::$option_name, 'bookingFormMainColor', '');
            if (empty($main_color)) {
                $main_color = '#0b75b4';
            }

            $main_color_dim = \sprintf('rgb(%s,%s,%s)', ...ColorHelper::darken($main_color, 10, false));

            return [$main_color, $main_color_dim];
        }

        public function updateWebsiteToLeads(): array|\WP_Error
        {
            return $this->leads->updateWebsite();
        }

        protected function setupMiddleWares(): array
        {
            return $this->registered['middlewares'] = [
                // Http::class => new Http()
            ];
        }
    }
}
