<?php

namespace JimsCustomerBooking;

use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use YahnisElsts\PluginUpdateChecker\v5\PucFactory;

class JimsVersioner
{
    public const UPDATE_MODE_STABLE = 'steady';
    public const UPDATE_MODE_EARLY = 'early';

    public static $update_endpoints = [
        'steady' => JIMS_CBP_INTERNAL_URL . '/api/downloads/jims-wpcb-steady/latest',
        'early' => JIMS_CBP_INTERNAL_URL . '/api/downloads/jims-wpcb-early/latest',
    ];

    public static function activate()
    {
        $option_name = StringHelper::getPrefixed('version', 'machine');
        $current_version = get_option($option_name);

        if ($current_version != JIMS_CBP_VERSION) {
            update_option($option_name, JIMS_CBP_VERSION, true);
        }
    }

    public static function updateCheck()
    {
        $options = OptionHelper::get(NativeFormSettingsPage::$option_name, []);
        $mode = self::UPDATE_MODE_STABLE;

        if (isset($options['auto_update']) && $options['auto_update'] === '1') {
            $mode = self::UPDATE_MODE_EARLY;
        }

        self::check($mode);
    }

    /**
     * @param string $mode
     */
    public static function check($mode = self::UPDATE_MODE_STABLE)
    {
        return PucFactory::buildUpdateChecker(
            self::$update_endpoints[$mode],
            JIMS_CBP_FILE, // Full path to the main plugin file or functions.php.
            self::getName($mode),
            3,
            self::getName($mode) . '-info'
        );
    }

    /**
     * @param mixed $mode
     *
     * @return null|array|string|string[]
     */
    public static function getName($mode = self::UPDATE_MODE_STABLE)
    {
        $name = StringHelper::getPrefix();
        $name .= '-' . $mode;

        //        if($mode == self::UPDATE_MODE_EARLY){
        //            $name .= '-early';
        //        }
        return $name;
    }
}
